% SU.PRO: Abstractions about structural units.
% Author: Frans Meijs, 1992


entity(E) :- structuralUnit(E).


entity__e_entity(E1,E2) :-
   behaviouralUnit__e_su(E1,E2).
entity__e_entity(E1,E2) :-
   structuralUnit__e_su(E1,E2).


structuralUnit__e_su(SU1,SU2) :-
   structuralUnit__isPrivateElementOf_su(SU1,SU2).
structuralUnit__e_su(SU1,SU2) :-
   structuralUnit__isShareableElementOf_su(SU1,SU2).


structuralUnit__isPrivateElementOf_su(SU1,SU2) :-
   structuralUnit__isExplicitlyPrivateElementOf_su(SU1,SU2).
structuralUnit__isPrivateElementOf_su(SU1,SU2) :-
   nonvar(SU1),
   structuralUnit__isExplicitlyPrivateElementOf_su(SU1,SU3),
   structuralUnit__isPrivateElementOf_su(SU3,SU2).
structuralUnit__isPrivateElementOf_su(SU1,SU2) :-
   var(SU1), nonvar(SU2),
   structuralUnit__isExplicitlyPrivateElementOf_su(SU3,SU2),
   structuralUnit__isPrivateElementOf_su(SU1,SU3).

structuralUnit__isShareableElementOf_su(SU1,SU2) :-
   structuralUnit__isExplicitlyShareableElementOf_su(SU1,SU2).
   
eachInstanceOfClass__isExplicitlyElementOf_set(SV,SU) :-
   stateVariable__e_su(SV,SU).


stateVariable__e_su(SV,SU) :-
   stateVariable__isPrivateElementOf_su(SV,SU).
stateVariable__e_su(SV,SU) :-
   stateVariable__isShareableElementOf_su(SV,SU).
stateVariable__e_su(SV,SU) :-
   stateVariable__isInputElementOf_su(SV,SU).
stateVariable__e_su(SV,SU) :-
   stateVariable__isOutputElementOf_su(SV,SU).


stateVariable__e_bu(SV,BU) :-
   stateVariable__isShareableElementOf_bu(SV,BU).
stateVariable__e_bu(SV,BU) :-
   stateVariable__isInputElementOf_bu(SV,BU).
stateVariable__e_bu(SV,BU) :-
   stateVariable__isOutputElementOf_bu(SV,BU).
                                         % Note 2.


stateVariable__isShareableElementOf_bu(SV,BU) :-
   stateVariable__isPrivateElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).   
stateVariable__isShareableElementOf_bu(SV,BU) :-
   stateVariable__isShareableElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).


stateVariable__isInputElementOf_bu(SV,BU) :-
   nonvar(SV),
   stateVariable__isInputElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).
stateVariable__isInputElementOf_bu(SV,BU) :-
   var(SV),nonvar(BU),
   behaviouralUnit__e_su(BU,SU),
   stateVariable__isInputElementOf_su(SV,SU).
stateVariable__isInputElementOf_bu(SV,BU) :-
   nonvar(SV),
   stateVariable__isPrivateElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).
stateVariable__isInputElementOf_bu(SV,BU) :-
   var(SV),nonvar(BU),
   behaviouralUnit__e_su(BU,SU),
   stateVariable__isPrivateElementOf_su(SV,SU).
   

stateVariable__isOutputElementOf_bu(SV,BU) :-
   nonvar(SV),
   stateVariable__isOutputElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).   
stateVariable__isOutputElementOf_bu(SV,BU) :-
   var(SV),nonvar(BU),
   behaviouralUnit__e_su(BU,SU),
   stateVariable__isOutputElementOf_su(SV,SU).      % Note 2.
stateVariable__isOutputElementOf_bu(SV,BU) :-
   nonvar(SV),
   stateVariable__isPrivateElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).
stateVariable__isOutputElementOf_bu(SV,BU) :-
   var(SV),nonvar(BU),
   behaviouralUnit__e_su(BU,SU),
   stateVariable__isPrivateElementOf_su(SV,SU).


stateVariable__isOutputElementOf_su(SV,SU) :-
   nonvar(SV),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2), % Note 1.
   structuralUnit__isOutputElementOf_su(SU2,SU).
stateVariable__isOutputElementOf_su(SV,SU) :-
   var(SV),nonvar(SU),
   structuralUnit__isOutputElementOf_su(SU2,SU),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2).


stateVariable__isPrivateElementOf_su(SV,SU) :-
   stateVariable__isExplicitlyPrivateElementOf_su(SV,SU).
stateVariable__isPrivateElementOf_su(SV,SU) :-
   nonvar(SV),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2), % Note 1.
   structuralUnit__isPrivateElementOf_su(SU2,SU).
stateVariable__isPrivateElementOf_su(SV,SU) :-
   var(SV),nonvar(SU),
   structuralUnit__isPrivateElementOf_su(SU2,SU),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2).


stateVariable__isShareableElementOf_su(SV,SU) :-
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU).
stateVariable__isShareableElementOf_su(SV,SU) :-
   nonvar(SV),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2), % Note 1.
   structuralUnit__isShareableElementOf_su(SU2,SU).
stateVariable__isShareableElementOf_su(SV,SU) :-
   var(SV),nonvar(SU),
   structuralUnit__isShareableElementOf_su(SU2,SU),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2).


stateVariable__isInputElementOf_su(SV,SU) :-
   nonvar(SV),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2), % Note 1.
   structuralUnit__isInputElementOf_su(SU2,SU).
stateVariable__isInputElementOf_su(SV,SU) :-
   var(SV),nonvar(SU),
   structuralUnit__isInputElementOf_su(SU2,SU),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2).



% Notes:

% 1.  NOT "_e__", because that would cause infinite recursion.
% 2. All "private/input/output" defs involving both SU-s and BU-s should
%     be confined to this file.
