% MSGV.PRC

contradiction(['Contrad. class MSGV1: Message variable', MV,
    'has no unique signalling event variable']) :- 
   o_verifyingLogMsg('MSGV1'),
   msgv(MV),
   propositionSet__has_cardinalNumber( sev__e_msgv(_, MV), N),
   N \= 1.

% PSVS.PRC

contradiction(['Contrad. class PSVS1: Passive State Variable Set', PSVS,
    'contains mapping class', MC]) :- 
   o_verifyingLogMsg('PSVS1'),
   psvs(PSVS),
   mappingClass__e_bu(MC, PSVS).


% SYMBOL.PRC


contradiction(
    ['Contrad. class SYMBOL1:', S,
    'represents ', C,
    'but is no symbol']) :-
   symbol__represents_concept(S, C),
   ~( symbol(S) ).


contradiction(
    ['Contrad. class SYMBOL11:', Symb,
    'represents both', S1, 'and', S2]) :-
   symbol__represents_concept(Symb, S1),
   symbol__represents_concept(Symb, S2),
   id__lt_id(S1, S2).
       % Spec of  * to <=1  association constraint.
       % (Homonyms violate this constraint.)



% SU.PRC: The constraints

contradiction(
   ['Contrad. class SU1',BU,
    'is element of more than one structural unit',SU]) :-
   o_verifyingLogMsg('SU1'),
   behaviouralUnit__e_su(BU,SU),
   propositionSet__has_cardinalNumber(
      behaviouralUnit__e_su(BU,_),
      N),
   N > 1. 


contradiction(
   ['Contrad. class SU2',SU,
    'has more than one behavioural unit:',DSM,DSM2]) :-
   o_verifyingLogMsg('SU2'),
   behaviouralUnit__e_su(DSM,SU),
   behaviouralUnit__e_su(DSM2,SU),
   id__lt_id(DSM,DSM2).
  

contradiction(
   ['Contrad. class SU11',BU,
    'seems to be behavioural unit of',SU,
    'but is not declared as such']) :-
   o_verifyingLogMsg('SU11'),
   behaviouralUnit__e_su(BU,SU),
   ~( behaviouralUnit(BU) ).


contradiction(
   ['Contrad. class SU12',SU1,
    'seems to be structural unit of',SU2,
    'but is not declared as such']) :-
   o_verifyingLogMsg('SU12'),
   structuralUnit__e_su(SU1,SU2),
   ~( structuralUnit(SU1) ). 

contradiction(
   ['Contrad. class SU21: Structural unit',S1,
   'is explicitly private element of both',S2,
   'and',S3]) :-
   o_verifyingLogMsg('SU21'),
   structuralUnit__isExplicitlyPrivateElementOf_su(S1,S2),
   structuralUnit__isExplicitlyPrivateElementOf_su(S1,S3),
   id__lt_id(S2,S3).

   
contradiction(
   ['Contrad. class SU22: Structural unit',S1,
   'is both private element and shareable element of',S2]) :-
   o_verifyingLogMsg('SU22'),
   structuralUnit__isPrivateElementOf_su(S1,S2),
   structuralUnit__isShareableElementOf_su(S1,S2).

contradiction(
   ['Contrad. class SU31: Structural unit',S1,
   'and',S2,
   'are both roots of the aggregation tree']) :-
   o_verifyingLogMsg('SU31'),
   structuralUnit(S1),
   ~( structuralUnit__e_su(S1,_) ),
   structuralUnit(S2),
   id__lt_id(S1,S2),
   ~( structuralUnit__e_su(S2,_) ).

contradiction(
   ['Contrad. class SU41: State variable',SV,
   'is both private element and shareable element of',S2]) :-
   o_verifyingLogMsg('SU41'),
   stateVariable__isPrivateElementOf_su(SV,S2),
   stateVariable__isShareableElementOf_su(SV,S2).

contradiction(
   ['Contrad. class SU42: State variable',SV,
   'is both private element and input element of',S2]) :-
   o_verifyingLogMsg('SU42'),
   stateVariable__isPrivateElementOf_su(SV,S2),
   stateVariable__isInputElementOf_su(SV,S2).

contradiction(
   ['Contrad. class SU43: State variable',SV,
   'is both private element and output element of',S2]) :-
   o_verifyingLogMsg('SU43'),
   stateVariable__isPrivateElementOf_su(SV,S2),
   stateVariable__isOutputElementOf_su(SV,S2).

contradiction(
   ['Contrad. class SU51: Structural unit',SU1,
    'may be enabled simultaneously by',FSM1,
    'and',FSM2]) :-
   o_verifyingLogMsg('SU51'),
   ms__enables_su(MS1,SU1),
   ms__enables_su(MS2,SU1),
   id__lt_id(MS1,MS2),

   ms__isAssociatedWith_fsm(MS1,FSM1),
   behaviouralUnit__e_su(FSM1,SU2),
   ms__enables_su(MS3,SU2),

   ms__isAssociatedWith_fsm(MS2,FSM2),
   FSM1 \= FSM2,
   behaviouralUnit__e_su(FSM2,SU3),
   ms__enables_su(MS3,SU3).
               % Closure To Be Defined.


% STATE.PRC: The constraints.

contradiction(['Contrad. class STATE1: State',S,
               'is also a subclass']) :-
   o_verifyingLogMsg('STATE1'),
   class(S),
   exists( state__e_entity(S,_) ).   % An individual state cannot be 
                                     % specified on these two levels
                                     % of abstraction at a time.
                                     
% SORT.PRC: Constraints.

contradiction(
   ['Contrad. class SORT1',D,
    'is no element of the hierarchy of sort inclusions']) :-
   o_verifyingLogMsg('SORT1'),
   sort(D),
   D \== ['SortOf','states'],
   ~( sort__isExplicitlyElementOf_sort(D,_) ).


contradiction(
   ['Contrad. class SORT3',D,
   'seems to be a sort of elem.states but is not declared as such']) :-
   o_verifyingLogMsg('SORT3'),
   elemState__e_sort(_,D),
   ~( sort(D) ).


contradiction(
   ['Contrad. class SORT4',D,
   'seems to be a sort of entities but is not declared as such']) :-
   o_verifyingLogMsg('SORT4'),
   entity__e_sort(_,D),
   ~( sort(D) ).



%:- consult('int.pro').
%:- consult('nat.pro').




% NMC.PRC

contradiction(['Contrad. class NMC1', MC,
   'maps from more than two variables;']) :-
   o_verifyingLogMsg('NMC1'),
   numericMc__mapsFrom_numericSv(MC, _),
   propositionSet__has_cardinalNumber(
     numericMc__mapsFrom_numericSv(MC, _),
     N),
   N > 2.

contradiction(['Contrad. class NMC2', MC,
   'maps to more than one variable;']) :-
   o_verifyingLogMsg('NMC2'),
   numericMc__mapsTo_numericSv(MC,_),
   propositionSet__has_cardinalNumber(
     numericMc__mapsTo_numericSv(MC, _),
     N),
   N > 1.

contradiction(['Contrad. class NMC11', NMC,
   'maps from non-numeric variable', NSV]) :-
   o_verifyingLogMsg('NMC11'),
   numericMc__mapsFrom_numericSv(NMC, NSV),
   ~( stateVariable__isAssociatedWith_sort(
                                          NSV, ['SortOf', 'numbers']) ).

contradiction(['Contrad. class NMC12', NMC,
   'maps to non-numeric variable', NSV]) :-
   o_verifyingLogMsg('NMC12'),
   numericMc__mapsTo_numericSv(NMC, NSV),
   ~( stateVariable__isAssociatedWith_sort(
                                          NSV, ['SortOf', 'numbers']) ).


contradiction(['Contrad. class NMC21', NSV,
   'has a non-alphanumeric unit of measure', UM]) :-
   o_verifyingLogMsg('NMC21'),
   unitOfMeasure__isAssociatedWith_numericSv(UM, NSV),
   ~( chars(UM) ).


contradiction(['Contrad. class NMC22', NMC,
   'maps to', NSV,
   'which has a pre-specified unit of measure', UM]) :-
   o_verifyingLogMsg('NMC22'),
   unitOfMeasure__isAssociatedWith_numericSv(UM, NSV),
   numericMc__mapsTo_numericSv(NMC, NSV).
   


% BMC.PRC

contradiction(['Contrad. class BMC1', X,
   'maps from more than two variables']) :-
   booleanMc__mapsFrom_booleanSv(X, _),
   propositionSet__has_cardinalNumber(
      booleanMc__mapsFrom_booleanSv(X, _),
      N),
   N > 2.

contradiction(['Contrad. class BMC2', X,
   'maps to more than one variable']) :-
   booleanMc__mapsTo_booleanSv(X, _),
   propositionSet__has_cardinalNumber(
      booleanMc__mapsTo_booleanSv(X, _),
      N),
   N > 1.

contradiction(['Contrad. class BMC11', BMC,
   'maps from non-boolean variable', BSV]) :-
   booleanMc__mapsFrom_booleanSv(BMC, BSV),
   ~(stateVariable__isAssociatedWith_sort(
                                 BSV, ['SortOf', 'booleanStates']) ).

contradiction(['Contrad. class BMC12', BMC,
   'maps to non-boolean variable', BSV]) :-
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   ~(stateVariable__isAssociatedWith_sort(
                                 BSV, ['SortOf', 'booleanStates']) ).


% ESMC.PRC: The constraints.

contradiction(['Contrad. class ESMC1',X,
   'does not map from a state variable']) :-
   o_verifyingLogMsg('ESMC1'),
   exists( elemStateMappingClass__mapsTo_stateVariable(X,_) ),
   ~( elemStateMappingClass__mapsFrom_stateVariable(X,_) ).


contradiction(['Contrad. class ESMC2',X,
   'does not map to a state variable']) :-
   o_verifyingLogMsg('ESMC1'),
   exists( elemStateMappingClass__mapsFrom_stateVariable(X,_) ),
   ~( elemStateMappingClass__mapsTo_stateVariable(X,_) ).


contradiction(['Contrad. class ESMC3',ESMC,
   'maps from state variable',ESV,
   'associated to sort',S,
   'which is no subset of the sort of elementaryStates']) :-
   o_verifyingLogMsg('ESMC3'),
   elemStateMappingClass__mapsFrom_stateVariable(ESMC,ESV),
   stateVariable__isAssociatedWith_sort(ESV,S),
   S \= ['SortOf','elementaryStates'],
   ~( sort__e_sort(S,['SortOf','elementaryStates']) ).


contradiction(['Contrad. class ESMC4',ESMC,
   'maps to state variable',ESV,
   'associated to sort',S,
   'which is no subset of the sort of elementaryStates']) :-
   o_verifyingLogMsg('ESMC4'),
   elemStateMappingClass__mapsTo_stateVariable(ESMC,ESV),
   stateVariable__isAssociatedWith_sort(ESV,S),
   S \= ['SortOf','elementaryStates'],
   ~( sort__e_sort(S,['SortOf','elementaryStates']) ).

% EMC.PRC: The constraints.

contradiction(['Contrad. class EMC1',X,
   'maps from more than two state variables, one being',Y]) :-
   entityMappingClass__mapsFrom_stateVariable(X,Y),
   o_verifyingLogMsg('EMC1'),
   propositionSet__has_cardinalNumber(
      entityMappingClass__mapsFrom_stateVariable(X,_),
      N),
   N > 2.


contradiction(['Contrad. class EMC2',X,
   'maps to more than one state variable, one being',Y]) :-
   entityMappingClass__mapsTo_stateVariable(X,Y),
   o_verifyingLogMsg('EMC2'),
   propositionSet__has_cardinalNumber(
      entityMappingClass__mapsTo_stateVariable(X,_),
      N),
   N > 1.


contradiction(['Contrad. class EMC3',EMC,
   'maps from state variable',ESV,
   'associated to sort',S,
   'which is no subset of the sort of entities']) :-
   o_verifyingLogMsg('EMC3'),
   entityMappingClass__mapsFrom_stateVariable(EMC,ESV),
   stateVariable__isAssociatedWith_sort(ESV,S),
   S \= ['SortOf','entities'],
   ~( sort__e_sort(S,['SortOf','entities']) ).


contradiction(['Contrad. class EMC4',EMC,
   'maps to state variable',ESV,
   'associated to sort',S,
   'which is no subset of the sort of entities']) :-
   o_verifyingLogMsg('EMC4'),
   entityMappingClass__mapsTo_stateVariable(EMC,ESV),
   stateVariable__isAssociatedWith_sort(ESV,S),
   S \= ['SortOf','entities'],
   ~( sort__e_sort(S,['SortOf','entities']) ).

% ASSIGNM.PRC


contradiction(['Contrad. class AC1', AC, SV]) :-
   o_verifyingLogMsg('AC1'),
   assignmentClass__mapsFrom_sv(AC, SV),
   propositionSet__has_cardinalNumber(
      assignmentClass__mapsFrom_sv(AC, _),
      N),
   N > 1.

contradiction(['Contrad. class AC2', AC, SV]) :-
   o_verifyingLogMsg('AC2'),
   assignmentClass__mapsTo_sv(AC, SV),
   propositionSet__has_cardinalNumber(
      assignmentClass__mapsTo_sv(AC, _),
      N),
   N > 1.

contradiction(['Contrad. class AC11', AC,
   'has a co-domain', DS1,
   'which is not a subset of its domain', DS2]) :-
   o_verifyingLogMsg('AC11'),
   assignmentClass__mapsFrom_sv(AC, SV1),
   assignmentClass__mapsTo_sv(AC, SV2),
   stateVariable__isAssociatedWith_sort(SV1, DS1),
   stateVariable__isAssociatedWith_sort(SV2, DS2),
   ~( concept__e_set(DS1, DS2) ).


% CC.PRC

contradiction(['Contrad. class CC1', X,
   'maps from more than two variables;']) :-
   comparisonClass__mapsFrom_sv(X, _),
   propositionSet__has_cardinalNumber(
      comparisonClass__mapsFrom_sv(X, _),
      N),
   N > 2.

contradiction(['Contrad. class CC2', X,
   'maps to more than one variable;']) :-
   comparisonClass__mapsTo_booleanSv(X, _),
   propositionSet__has_cardinalNumber(
      comparisonClass__mapsTo_booleanSv(X, _),
      N),
   N > 1.

contradiction(['Contrad. class CC11', CC,
   'maps to non-boolean variable', SV]) :-
   comparisonClass__mapsTo_booleanSv(CC, SV),
   ~( stateVariable__isAssociatedWith_sort(
                                 SV, ['SortOf', 'booleanStates']) ).


% MAPPINGC.PRC: The constraints.
% (c) Author: Frans A. Meijs, 1992.

contradiction(['Contrad. class MAPPINGC1: state variable',SV,
      'has more than one instantaneous state']) :-
   o_verifyingLogMsg('MAPPINGC1'),
   class__isSubclassOf_class(SV,'StateVariable'),
   propositionSet__has_cardinalNumber(
        state__isExplicitlyInstanceOf_stateVariable(_,SV),
        N),
   N > 1.  
 
contradiction(['Contrad. class MAPPINGC2: sort',S,
      'is associated with state variable',SV,
      'but is not declared as such']) :-
   o_verifyingLogMsg('MAPPINGC2'),
   stateVariable__isExplicitlyAssociatedWith_sort(SV,S),
   ~( sort(S) ).  
 
contradiction(['Contrad. class MAPPINGC11: Instantaneous state',S,
      'of state variable',SV,
      'is not element of the sort',DS]) :- 
   o_verifyingLogMsg('MAPPINGC11'),
   state__isExplicitlyInstanceOf_stateVariable(S,SV),
   stateVariable__isAssociatedWith_sort(SV,DS),
   ~( concept__e_set(S,DS) ).

contradiction(['Contrad. class MAPPINGC21: State variable',SV,
      'in mapping class',MC,
      'is not associated with any sort']) :- 
   o_verifyingLogMsg('MAPPINGC21'),
   eachInstanceOfMc__mapsFromAnInstanceOf_sv(MC,SV),
   ~( class__isExplicitlySubclassOf_class(_,SV) ),
   ~( stateVariable__isExplicitlyAssociatedWith_sort(SV,_) ).

contradiction(['Contrad. class MAPPINGC22: State variable',SV,
      'in mapping class',MC,
      'is not associated with any sort']) :- 
   o_verifyingLogMsg('MAPPINGC22'),
   eachInstanceOfMc__mapsToAnInstanceOf_sv(MC,SV),
   ~( stateVariable__isExplicitlyAssociatedWith_sort(SV,_) ).

contradiction(['Contrad. class MAPPINGC41: Mapping class',MC,
      'maps from superclass',SV]) :- 
   o_verifyingLogMsg('MAPPINGC41'),
   eachInstanceOfMc__mapsFromAnInstanceOf_sv(MC,SV),
   exists( class__isExplicitlySubclassOf_class(_,SV) ).

contradiction(['Contrad. class MAPPINGC42: Mapping class',MC,
      'maps to superclass',SV]) :- 
   o_verifyingLogMsg('MAPPINGC42'),
   eachInstanceOfMc__mapsToAnInstanceOf_sv(MC,SV),
   exists( class__isExplicitlySubclassOf_class(_,SV) ).

% TREE.PRC

contradiction(['Contrad. class','TREE1', TR,
   'has both', C1,
   'and', C2,
   'connecting to node', ND]) :-
   o_verifyingLogMsg('TREE1'),
   tree(TR),
   connection__e_network(C1, TR),
   connection__connectsTo_node(C1, ND),
   connection__e_network(C2, TR),
   connection__connectsTo_node(C2, ND),
   id__lt_id(C1, C2).


% HIERARCH.PRC: The constraints.


contradiction(['Contrad. class','HIERARCH1',H,
   'contains a unit cycle connection',C]) :-
   o_verifyingLogMsg('HIERARCH1'),
   hierarchy(H),
   connection__e_network(C,H),
   connection__connectsFrom_node(C,N),
   connection__connectsTo_node(C,N).


contradiction(
   ['Contrad. class','HIERARCH2',HR,
   'has only a subset',IHR,
   'as hierarchy']) :-
   o_verifyingLogMsg('HIERARCH2'),
   hierarchy(HR),
      o_chars('Hierarchy '),     % Detailed log.
      o_chars(HR),
      o_chars('... '), % o_eor cannot be used here because it is no IPL term.
   referenceConnection__e_network(C,HR),
   C_L = [C],
   setOfConnections__e_implicitHierarchy__in_diNet(C_L,IHR,HR),
   list__has_length(IHR,L1),
   propositionSet__has_cardinalNumber(
                                 connection__e_network(_,HR),
                                 L2
                                 ),
   L1 < L2.


% ADN_PRC: Constraints.

contradiction(['Contrad. class ADN1, connection', C,
   'in', ADN,
   'has a non-numeric cost attribute', TC]) :-
   o_verifyingLogMsg('ADN1'),
   attrDiNet(ADN),
   connection__e_network(C, ADN),
   cost__isAttributeOf_connection(TC, C),
   ~( number(C) ).

contradiction(['Contrad. class ADN2, connection', C,
   'in', ADN,
   'has more than one cost attribute']) :-
   o_verifyingLogMsg('ADN2'),
   attrDiNet(ADN),
   connection__e_network(C, ADN),
   cost__isAttributeOf_connection(TC1, C),
   cost__isAttributeOf_connection(TC2, C),
   TC1 < TC2.
       % Spec of  1 to 1  association constraint.

% DI_NET.PRC

contradiction(['Contrad. class DI_NET1', C,
   'connects from more than node', N]) :-
   o_verifyingLogMsg('DI_NET1'),
   connection__connectsFrom_node(C, N),
   propositionSet__has_cardinalNumber(
      connection__connectsFrom_node(C, _),
      NN),
   NN > 1.
                                % * to <= 1  association constraint.


contradiction(['Contrad. class DI_NET2', C,
   'connects to more than node', N]) :-
   o_verifyingLogMsg('DI_NET2'),
   connection__connectsTo_node(C, N),
   propositionSet__has_cardinalNumber(
      connection__connectsTo_node(C, _),
      NN),
   NN > 1.
                                 % * to <= 1  association constraint.

% CONN_NET.PRC: The constraints.
% Author: Frans Meijs, 4 sep 1996.

contradiction(['Contrad. class','CONN_NET1:', CN,
   'has only a subset', ICN,
   'as connected net']) :-
   o_verifyingLogMsg('CONN_NET1'),
   connNet(CN),
      o_chars('Conn.net '),     % Detailed log.
      o_chars(CN),
      o_chars('... '), % o_eor cannot be used here because it is no IPL term.
   referenceConnection__e_network(C, CN),
   C_L = [C],
   setOfConnections__e_implicitConnNet__in_network(C_L, ICN, CN),
   list__has_length(ICN, L1),
   propositionSet__has_cardinalNumber(
                                 connection__e_network(_, CN),
                                 L2
                                 ),
   L1 < L2.

% NETWORK.PRC

contradiction(['Contrad. class NETWORK1', C,
   'is connection in both', N1,
   'and', N2]) :-
   o_verifyingLogMsg('NETWORK1'),
   connection__e_network(C, N1),
   connection__e_network(C, N2),
   id__lt_id(N1, N2).
       % Spec of  * to <=1  association constraint.


contradiction(['Contrad. class NETWORK2: Connection', C,
   'in network', N,
   'has improper number of nodes']) :-
   o_verifyingLogMsg('NETWORK2'),
   connection__e_network(C, N),
   propositionSet__has_cardinalNumber(
      connection__connects_node(C, _),
      NN),
   NN \== 2.
       % Spec of  2 to 1  association constraint.


contradiction(['Contrad. class NETWORK3: Connection', C1,
   'and', C2,
   'are both reference in network', N]) :-
   o_verifyingLogMsg('NETWORK3'),
   referenceConnection__e_network(C1, N),
   referenceConnection__e_network(C2, N),
   id__lt_id(C1, C2).


% TQOSET.PRC: The constraints

contradiction(['Contrad. class TQOSET1', TQOS,
   'has both concepts',C1,C2,
   'immediately subordinate to',C3]) :-
   o_verifyingLogMsg('TQOSET1'),
   tqoSet(TQOS),
   concept__e_qoSet(C1,TQOS),
   concept__e_qoSet(C2,TQOS),
   concept__e_qoSet(C3,TQOS),
   concept__isExplicitlySubordinateTo_concept(C1,C3),
   concept__isExplicitlySubordinateTo_concept(C2,C3),
   id__lt_id(C1,C2).       % Suppress inverse solution.




% QOSET.PRC: The constraints

contradiction(['Contrad. class QOSET1', QOS,
   'has',C1,
   'with same rank as',C2]) :-
   o_verifyingLogMsg('QOSET1'),
   qoSet(QOS),
   concept__e_qoSet(C1,QOS),
   concept__e_qoSet(C2,QOS),
   concept__isExplicitlySubordinateTo_concept(C1,C2),
   concept__isExplicitlySubordinateTo_concept(C2,C1),
   id__lt_id(C1,C2).       % Suppress inverse solution.




% SET.PRC

contradiction(['Contrad. class SET1: Concept', X,
   'seems to be a set but is not declared as such']) :-
   o_verifyingLogMsg('SET1'),
   concept__isExplicitlyElementOf_set(_, X),
   ~( set(X) ).
      % This is the Referential Integrity Constraint
      % from data base theory, part 1.


contradiction(['Contrad. class SET2: Concept', X,
   'seems to be a set but is not declared as such']) :-
   o_verifyingLogMsg('SET2'),
   set__isExplicitlyElementOf_set(X, _),
   ~( set(X) ).
      % This is the Referential Integrity Constraint
      % from data base theory, part 2.

      % Sets are required to be specified explicitly. The 
      % alternative, the introduction of the specification  
      %         "set(S) :- set__isExplicitlyElementOf_set(S,_)." 
      % may cause duplicate solutions, and 
      % hence may increase verification time considerably.



% The contradiction "concept__e_set(X, X)." is not queried for, as this 
% takes N**2 time. Checks of this type have to be performed at the most
% specific levels.



contradiction(['Contrad. class SET4: Set', S,
               'is also a class']) :-
   o_verifyingLogMsg('SET4'),
   set(S),
   class(S).                        % An individual set cannot be 
                                    % specified on these two levels
                                    % of abstraction at the same time.



% CLASS.PRC

contradiction(['Contrad. class CLASS1: Concept', X,
   'seems to be a class but is not declared as such']) :-
   o_verifyingLogMsg('CLASS1'),
   class__isExplicitlySubclassOf_class(_, X),
   ~( class(X) ).
      % This is the first part of the Referential Integrity Constraint
      % from data base theory.


contradiction(['Contrad. class CLASS2: Concept', X,
   'seems to be a class but is not declared as such']) :-
   o_verifyingLogMsg('CLASS2'),
   class__isExplicitlySubclassOf_class(X, _),
   ~( class(X) ).
      % This is the second part of the Referential Integrity Constraint
      % from relational data base theory.
      % Classes are required to be specified explicitly. The 
      % alternative, the introduction of the specification  
      %    "class(C) :- class__isExplicitlyElementOf_class(C,_)." 
      % may cause duplicate solutions, and 
      % hence may increase verification time considerably.


contradiction(
   ['Contrad. class CLASS4', C1,
   'and', C2,
   'are both root in the classification tree']) :-
   o_verifyingLogMsg('CLASS4'),
   class(C1),
   ~( class__isExplicitlySubclassOf_class(C1, _) ),
   class(C2),
   id__lt_id(C1, C2),
   ~( class__isExplicitlySubclassOf_class(C2, _) ).

 

% CONCEPT.PRC


% - None

% CONCEPT.PRO: The theory with the highest level of abstraction.
% Author: Frans Meijs, 1990


concept(C) :- contradiction(C).        % For verification purposes.
concept(C) :- association(C).


association(A) :-
   nonvar(A),
   A = [C1|[C2]],
   concept__isAssociatedWith_concept(C1,C2).
association(A) :-
   var(A),
   concept__isAssociatedWith_concept(C1,C2),
   A = [C1,C2].


concept__isAssociatedWith_concept(C1,C2) :-
   concept__isExplicitlyAssociatedWith_concept(C1,C2).
         % Association is directed from subject noun to object noun, so
         %    concept__isExplicitlyAssociatedWith_concept(C2,C1) :-
         %                  concept__isAssociatedWith_concept(C1,C2).
         % is NOT true.
concept__isAssociatedWith_concept(C1,C2) :-
   concept__isExplicitlyAssociatedWith_concept(C1,C3),
   concept__isAssociatedWith_concept(C3,C2).
         % Because it is the most general closure, this clause is unrestricted w.r.t.
         % binding of C1 and C2.


%:- consult('concept.prc').
% CLASS.PRO: Abstractions about classes without explicit 
% identification of their instances.
% Author: Frans Meijs, 1990

concept(S) :- class(S).

concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   class__isSubclassOf_class(C1,C2).


class__isSubclassOf_class(C,C2) :- 
   class__isExplicitlySubclassOf_class(C,C2).
class__isSubclassOf_class(C1,C2) :-
   class__isExplicitlySubclassOf_class(C1,C3),
   C1 \= C3,            % Avoid infinite loop in case of erratic spec.
   class__isSubclassOf_class(C3,C2). 
               % It is assumed that there are no cycles (See CLASS.PRC).


association(A) :-
   nonvar(A),
   A = [C1|[C2]],
   class__isElementarySubclassOf_class(C1,C2).
association(A) :-
   var(A),
   class__isElementarySubclassOf_class(C1,C2),
   A = [C1,C2].
            % class__isElementarySubclassOf_class/2 is an 
            % auxiliary predicate.
            % For that reason it is not made subclass of 
            % concept__isExplicitlyAssociatedWith_concept/2.


class__isElementarySubclassOf_class(C1,C2) :-
   class__isSubclassOf_class(C1,C2),
   ~( class__isExplicitlySubclassOf_class(_,C1) ).



%:- consult('class.prc'). 
%:- consult('class.prp'). 


% SET.PRO: Abstractions about sets.
% Author: Frans Meijs, 1990


concept(C) :- set(C).


concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   set__intersects_set(C1,C2).
concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   concept__e_set(C1,C2).


concept__e_set(C,S) :-
   concept__isExplicitlyElementOf_set(C,S).
concept__e_set(C,S) :-
   nonvar(C),
   concept__isExplicitlyElementOf_set(C,S2),
   set__e_set(S2,S).
concept__e_set(C,S) :-
   var(C),nonvar(S),
   set__e_set(S2,S),
   concept__isExplicitlyElementOf_set(C,S2).


concept__isExplicitlyElementOf_set(C,S) :-
   eachInstanceOfClass__isExplicitlyElementOf_set(C,S).


set__e_set(S1,S2) :-
   set__isExplicitlyElementOf_set(S1,S2).
set__e_set(S1,S2) :-
   nonvar(S1),
   set__isExplicitlyElementOf_set(S1,S3),
   set__e_set(S3,S2).
set__e_set(S1,S2) :-
   var(S1), nonvar(S2),
   set__isExplicitlyElementOf_set(S3,S2),
   set__e_set(S1,S3).
                        % There are no cycles (See SET.PRC).                                    




%:- consult('set.prc').
% QOSET.PRO: Abstractions about quasi ordered sets.
% (c) Author: Frans A. Meijs, 1998

set(S) :- qoSet(S).

concept__e_set(C1,C2) :-
   concept__e_qoSet(C1,C2).
   
concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   concept__isSubordinateTo_concept(C1,C2).
            % Is equivalent to  isImmediatelySubordinateTo

concept__isSubordinateTo_concept(C1,C2) :-
   concept__isExplicitlySubordinateTo_concept(C1,C2).
concept__isSubordinateTo_concept(C1,C2) :-
   concept__isExplicitlySubordinateTo_concept(C1,C3),
   concept__isSubordinateTo_concept(C3,C2).
   

set__isExplicitlyElementOf_set(S1,S2) :-
   qoSet__e_qoSet(S1,S2).


qoSet__e_qoSet(QS1,QS2) :-
   concept__e_qoSet(C,QS2),
   qoSet__isExplicitlySubordinateTo_concept(QS1,C).



% TQOSET.PRO: Abstractions about totally quasi ordered sets.
% Author: Frans Meijs, 7 July 1998


set(S) :- tqoSetElement(S).
               % A tqoSetElement is more abstract than an entity.
               
qoSet(S) :- tqoSet(S).

concept__isExplicitlyElementOf_set(C,S) :-
   dimension__e_tqoSet(C,S).
concept__isExplicitlyElementOf_set(C,S) :-
   ordinalNumber__e_tqoSetElement(C,S).


concept__e_qoSet(S1,S2) :-
   tqoSetElement__e_tqoSet(S1,S2).

   

% COMBIN.PRO: Abstractions about combinations.
% Author: Frans Meijs, 1 jan 1999.


concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   tqoSet__isCombinationOutOf_tqoSetOfSorts(C1,C2).


tqoSet__isCombinationOutOf_tqoSetOfSorts(TS,TSS) :-
   TS = [E],
   TSS = [S],
   concept__e_set(E,S).
tqoSet__isCombinationOutOf_tqoSetOfSorts(TS,TSS) :-
   TS = [H|T],
   TSS = [SH|ST],
   nonvar(SH),
   concept__e_set(H,SH),
   tqoSet__isCombinationOutOf_tqoSetOfSorts(T,ST).

% NETWORK.PRO: Abstractions about networks.


concept(Z) :- node(Z).
set(Z) :- network(Z).

association(Z) :- connection(Z).  % Cannot be a set because it may have 
                                  % classes as specializations.

concept__isExplicitlyElementOf_set(C, N) :-
   connection__e_network(C, N).

connection__e_network(C, N) :-
   referenceConnection__e_network(C, N).

concept__isExplicitlyAssociatedWith_concept(C, Node) :-
   connection__connects_node(C, Node).


%:- consult('network.prc').



% CONN_NET.PRO: Abstractions about connected networks.


network(CN) :- connNet(CN).


setOfConnections__e_implicitConnNet__in_network(CS, ICN, N) :-
   nonvar(CS), nonvar(N),
   list__has_length(CS, L),
   L > 1,
   ~(
     setOfConnections__connectsTo_connection__in_network(CS, _, N)
    ),
   ICN = CS,
   !.                                       % Solution is sufficient.
setOfConnections__e_implicitConnNet__in_network(CS, ICN, N) :-
   nonvar(CS), nonvar(N),
   setOfConnections__connectsTo_connection__in_network(CS, C, N),
   CS2 = [C| CS],
   setOfConnections__e_implicitConnNet__in_network(CS2, ICN, N).


setOfConnections__connectsTo_connection__in_network(CS, C, N) :-
   nonvar(CS), nonvar(N),
   term__isMemberOf_list(C1, CS),
   connection__e_network(C1, N),        % Limitation of search space 
                                        % for ver. speed. (Compositional
                                        % semantics)
   connection__connects_node(C1, ND),
   connection__connects_node(C, ND),
   connection__e_network(C, N),
   C \== C1,
   ~( term__isMemberOf_list__v2(C, CS) ),
   !.                                       % Solution is sufficient.


%:- consult('conn_net.prc').

% DI_NET.PRO: Abstractions about directed networks.


connNet(Z) :- diNet(Z).

connection__connects_node(C, N) :-
   connection__connectsFrom_node(C, N).

connection__connects_node(C, N) :-
   connection__connectsTo_node(C, N).


%:- consult('di_net.prc').

% ADN.PRO: Abstractions about attributed directed networks.

diNet(ADN) :- attrDiNet(ADN).

concept__isExplicitlyAssociatedWith_concept(TC, C) :-
   cost__isAttributeOf_connection(TC, C).
      % Rationale: __isAttributeOf_ is not a specialization of __e_
      % because a connection is not a set, see Network.pro


%:- consult('adn.prc').
% HIERARCH.PRO: Abstractions about hierarchies.


diNet(H) :- hierarchy(H).


setOfConnections__e_implicitHierarchy__in_diNet(CS,IHR,N) :-
   nonvar(CS),nonvar(N),
   list__has_length(CS,L),
   L > 1,
   ~(
     setOfConnections__connectsTo_connection__in_diNet(CS,_,N)
    ),
   IHR = CS,
   !.                                       % Solution is sufficient.
setOfConnections__e_implicitHierarchy__in_diNet(CS,IHR,N) :-
   nonvar(CS),nonvar(N),
   setOfConnections__connectsTo_connection__in_diNet(CS,C,N),
   ~( connection__makesCycleInto_diNet(C,CS) ),
   CS2 = [C|CS],
   setOfConnections__e_implicitHierarchy__in_diNet(CS2,IHR,N).
setOfConnections__e_implicitHierarchy__in_diNet(CS,IHR,N) :-
   nonvar(CS),nonvar(N),
   setOfConnections__connectsTo_connection__in_diNet(CS,C,N),
   connection__makesCycleInto_diNet(C,CS),
   IHR = CS.
                   % The implicit hierarchy is not needed to be complete.


setOfConnections__connectsTo_connection__in_diNet(CS,C,N) :-
   nonvar(CS),nonvar(N),
   term__isMemberOf_list(C1,CS),
   connection__e_network(C1,N),       % Limitation of search space 
                                        % for ver. speed. (Compositional
                                        % semantics)
   connection__connectsTo_node(C1,ND),
   connection__connectsFrom_node(C,ND),
   connection__e_network(C,N),
   ~( term__isMemberOf_list__v2(C,CS) ).


connection__makesCycleInto_diNet(C,CS) :-
   nonvar(C),nonvar(CS),
   connection__connectsTo_node(C,ND2),
   connection__connectsFrom_node(C2,ND2),
   term__isMemberOf_list__v2(C2,CS),
   C2_L = [C2],
   path__connectedTo_connection__isPath(C2_L,C,_).
 

path__connectedTo_connection__isPath(P,C,P2) :-
   nonvar(P),nonvar(C),
   P2 = [C|P],
   P = [C1|_],
   connection__connectsTo_node(C1,ND),
   connection__connectsFrom_node(C,ND),
   !.                                        % Solution is sufficient.
path__connectedTo_connection__isPath(P1,C2,P2) :-
   nonvar(P1),nonvar(C2),
   P1 = [C1|_],
   connection__connectsTo_node(C1,ND),
   connection__connectsFrom_node(C3,ND),
   ~( term__isMemberOf_list__v2(C3,P1) ),
   path__connectedTo_connection__isPath([C3|P1],C2,P2).
   

% TREE.PRO: Abstractions about trees.


hierarchy(H) :- tree(H).


%:- consult('tree.prc').
%:- consult('$PROLOG/outtrac.pro').
%:- consult('$PROLOG/chars.pro').
%:- consult('$PROLOG/flatlist.pro').
%:- consult('$PROLOG/list.pro').
%:- consult('tests/tree.prp').

% STATE.PRO: Abstractions about states.


set(S) :- entity(S).

concept__isExplicitlyElementOf_set(C,S) :-
   state__e_entity(C,S).


state__e_entity(S,E) :-
   entity__e_entity(S,E).
state__e_entity(S,E) :-
   elemState__e_entity(S,E).



%:- consult('state.prc').
% MAPPINGC.PRO: Abstractions about mapping classes.
% Author: Frans Meijs, 1992


concept__isExplicitlyAssociatedWith_concept(C,S) :-
   class__isAssociatedWith_set(C,S).
concept__isExplicitlyAssociatedWith_concept(S,C) :-
   state__isExplicitlyAssociatedWith_class(S,C).
concept__isExplicitlyAssociatedWith_concept(M,MC) :-
   mapping__isExplicitlyInstanceOf_mappingClass(M,MC).
concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   state__isNextInstanceOf_stateVariable(C1,C2).
concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   eachInstanceOfMc__mapsFromAnInstanceOf_sv(C1,C2).
concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   eachInstanceOfMc__mapsToAnInstanceOf_sv(C1,C2).


class__isAssociatedWith_set(SV,DS) :-
   stateVariable__isAssociatedWith_sort(SV,DS).


stateVariable__isAssociatedWith_sort(SV,DS) :-
   stateVariable__isExplicitlyAssociatedWith_sort(SV,DS).
stateVariable__isAssociatedWith_sort(SV,DS) :-
   nonvar(SV),
   stateVariable__isExplicitlyAssociatedWith_sort(SV,DS2),
   sort__e_sort(DS,DS2),
   exists( sort(DS) ).
stateVariable__isAssociatedWith_sort(SV,DS) :-
   var(SV),nonvar(DS),
   exists( sort(DS) ),
   sort__e_sort(DS,DS2),
   stateVariable__isExplicitlyAssociatedWith_sort(SV,DS2).


state__isExplicitlyAssociatedWith_class(S,SV) :-
   state__i_stateVariable(S,SV).


state__isNextInstanceOf_stateVariable(IS,SV) :-
   nonvar(IS), nonvar(SV),
   retract( state__isExplicitlyInstanceOf_stateVariable(_,SV) ),
   assert( state__isExplicitlyInstanceOf_stateVariable(IS,SV) ).    
                                                % Notes 1 and 2.
                                    

% Notes:

% 1. The relation between "_i_" and "_isExplicitlyInstanceOf_"
%    is made by specific mapping classes.
% 2. Update of the specification of the instantaneous state.
%    This represents the operational semantics of mapping classes.
% 3. Associations with diNets have to be made in more specific 
%    theories, dependent on the type of information to be verified.

%:- consult('mappingc.prc').
%:- consult('mappingc.prp').


% VECTORV.PRO: Abstractions about vector variables.
% Author: Frans Meijs, 25 dec 1998.


tqoSetElement(TQSE) :- vectorVariableElement(TQSE).
tqoSet(TQS) :- vectorVariable(TQS).


tqoSetElement__e_tqoSet(TQSE,TQS) :-
   vectorVariableElement__e_vectorVariable(TQSE,TQS).


eachInstanceOfClass__isExplicitlyElementOf_set(IC,S) :-
   stateVariable__e_vectorVariableElement(IC,S).


concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   vectorVariable__isAssociatedWith_sort(C1,C2).
   

vectorVariable__isAssociatedWith_sort(VV,S) :-
   nonvar(VV),
   ~( qoSet__e_qoSet(_,VV) ),
   vectorVariableElement__e_vectorVariable(VVE,VV),
   vectorVariableElement__isAssociatedWith_sort(VVE,S2),
   S = [S2].
vectorVariable__isAssociatedWith_sort(VV,S) :-
   nonvar(VV),
   qoSet__e_qoSet(VV2,VV),
   vectorVariableElement__e_vectorVariable(VVE,VV),
   vectorVariableElement__isAssociatedWith_sort(VVE,S2),
   S = [S2|S3],
   vectorVariable__isAssociatedWith_sort(VV2,S3).
vectorVariable__isAssociatedWith_sort(VV,S) :-
   var(VV),nonvar(S),
   S = [S2],
   vectorVariableElement__isAssociatedWith_sort(VVE,S2),
   vectorVariableElement__e_vectorVariable(VVE,VV),
   ~( qoSet__e_qoSet(_,VV) ).
vectorVariable__isAssociatedWith_sort(VV,S) :-
   var(VV),nonvar(S),
   S = [S2|S3],
   vectorVariableElement__isAssociatedWith_sort(VVE,S2),
   vectorVariableElement__e_vectorVariable(VVE,VV),
   qoSet__e_qoSet(VV2,VV),
   vectorVariable__isAssociatedWith_sort(VV2,S3).


vectorVariableElement__isAssociatedWith_sort(VVE,S) :-
   stateVariable__e_vectorVariableElement(SV,VVE),
   stateVariable__isAssociatedWith_sort(SV,S).





% BU.PRO: Abstractions about behavioural units.
% Author: Frans Meijs, 1992

diNet(DN) :- behaviouralUnit(DN).


state__e_entity(S,BU) :-
   state__e_bu(S,BU).


state__e_bu(ST,BU) :-
   nonvar(ST),
   state__isExplicitlyInstanceOf_stateVariable(ST,SV),
   stateVariable__isPrivateElementOf_bu(SV,BU).
state__e_bu(ST,BU) :-
   var(ST),nonvar(BU),
   stateVariable__isPrivateElementOf_bu(SV,BU),
   state__isExplicitlyInstanceOf_stateVariable(ST,SV).


concept__isExplicitlyElementOf_set(M,BU) :-
   mapping__e_bu(M,BU).


eachInstanceOfClass__isExplicitlyElementOf_set(IC,S) :-
   stateVariable__e_bu(IC,S).
eachInstanceOfClass__isExplicitlyElementOf_set(IC,S) :-
   mappingClass__e_bu(IC,S).


stateVariable__e_bu(SV,BU) :-
   stateVariable__isPrivateElementOf_bu(SV,BU).       % Note 1.


stateVariable__isPrivateElementOf_bu(SV,BU) :-
   nonvar(SV),
   stateVariable__e_vectorVariableElement(SV,VE),
   vectorVariableElement__e_vectorVariable(VE,SVCTR),
   stateVectorVariable__e_bu(SVCTR,BU).
stateVariable__isPrivateElementOf_bu(SV,BU) :-
   var(SV),nonvar(BU),
   stateVectorVariable__e_bu(SVCTR,BU),
   vectorVariableElement__e_vectorVariable(VE,SVCTR),
   stateVariable__e_vectorVariableElement(SV,VE).
               % This allows specs including vectors as well as excluding them.


entity__e_entity(E1,E2) :-
   stateVector__e_bu(E1,E2).
%entity__e_entity(E1,E2) :-
%   interfaceVector__e_bu(E1,E2).       % For documentation only.
      

%:- consult('bu.prc').

% Notes:

% 1. The other stateVariable__e_bu/2 defs are in SU.PRO
% ASSIGNM.PRO: Abstractions about assignment classes.
% (c) Author: Frans A. Meijs, 1992.

eachInstanceOfMc__mapsFromAnInstanceOf_sv(AC, SV) :-
   assignmentClass__mapsFrom_sv(AC, SV).

eachInstanceOfMc__mapsToAnInstanceOf_sv(AC, SV) :-
   assignmentClass__mapsTo_sv(AC, SV).

state__i_stateVariable(S, SV2) :-
   nonvar(SV2),
   assignmentClass__mapsTo_sv(AC, SV2),
   assignmentClass__mapsFrom_sv(AC, SV1),
   state__isExplicitlyInstanceOf_stateVariable(S, SV1),
   state__isNextInstanceOf_stateVariable(S, SV2).


%:- consult('assignm.prc').

% CC.PRO: Abstractions about comparison classes.



eachInstanceOfMc__mapsFromAnInstanceOf_sv(CC, SV) :-
   comparisonClass__mapsFrom_sv(CC, SV).

eachInstanceOfMc__mapsToAnInstanceOf_sv(CC, SV) :-
   comparisonClass__mapsTo_booleanSv(CC, SV).

state__i_stateVariable(BS, BSV) :-
   booleanState__i_booleanSv(BS, BSV).


booleanState__i_booleanSv(BS, SV) :-
   nonvar(SV),
   comparisonClass__mapsTo_booleanSv(CC, SV),
   comparisonClass__mapsFrom_sv(CC, SV1),
   comparisonClass__mapsFrom_sv(CC, SV2),
   state__isExplicitlyInstanceOf_stateVariable(S1, SV1),
   state__isExplicitlyInstanceOf_stateVariable(S2, SV2),
   class__isExplicitlySubclassOf_class(CC, '='),
   id__lt_id(S1, S2),
   BS = 'FALSE',
   state__isNextInstanceOf_stateVariable(BS, SV).
booleanState__i_booleanSv(BS, SV) :-
   nonvar(SV),
   comparisonClass__mapsTo_booleanSv(CC, SV),
   comparisonClass__mapsFrom_sv(CC, SV1),
   comparisonClass__mapsFrom_sv(CC, SV2),
   id__lt_id(SV1, SV2),
   state__isExplicitlyInstanceOf_stateVariable(S1, SV1),
   state__isExplicitlyInstanceOf_stateVariable(S2, SV2),
   class__isExplicitlySubclassOf_class(CC, '='),
   S1 == S2,
   BS = 'TRUE',
   state__isNextInstanceOf_stateVariable(BS, SV).
booleanState__i_booleanSv(BS, SV) :-
   nonvar(SV),
   comparisonClass__mapsTo_booleanSv(CC, SV),
   comparisonClass__mapsFrom_sv(CC, SV1),
   comparisonClass__mapsFrom_sv(CC, SV2),
   id__lt_id(SV1, SV2),
   state__isExplicitlyInstanceOf_stateVariable(S1, SV1),
   state__isExplicitlyInstanceOf_stateVariable(S2, SV2),
   class__isExplicitlySubclassOf_class(CC, '~='),
   S1 == S2,
   BS = 'FALSE',
   state__isNextInstanceOf_stateVariable(BS, SV).
booleanState__i_booleanSv(BS, SV) :-
   nonvar(SV),
   comparisonClass__mapsTo_booleanSv(CC, SV),
   comparisonClass__mapsFrom_sv(CC, SV1),
   comparisonClass__mapsFrom_sv(CC, SV2),
   state__isExplicitlyInstanceOf_stateVariable(S1, SV1),
   state__isExplicitlyInstanceOf_stateVariable(S2, SV2),
   class__isExplicitlySubclassOf_class(CC, '~='),
   id__lt_id(S1, S2),
   BS = 'TRUE',
   state__isNextInstanceOf_stateVariable(BS, SV).



%:- consult('cc.prc').
%:- consult('cc.prp').

% EMC.PRO: Abstractions about entity mapping classes.


state__i_stateVariable(E,SV) :-
   entity__i_stateVariable(E,SV).

eachInstanceOfMc__mapsFromAnInstanceOf_sv(EMC,SV) :-
   entityMappingClass__mapsFrom_stateVariable(EMC,SV).

eachInstanceOfMc__mapsToAnInstanceOf_sv(EMC,SV) :-
   entityMappingClass__mapsTo_stateVariable(EMC,SV).


entity__i_stateVariable(S,SV) :-
   entityMappingClass__mapsTo_stateVariable(EMC,SV),
   class__isExplicitlySubclassOf_class(EMC,'intersection'),
   entityMappingClass__mapsFrom_stateVariable(EMC,SV1),
   entityMappingClass__mapsFrom_stateVariable(EMC,SV2),
   id__lt_id(SV1,SV2),
   entity__isExplicitlyInstanceOf_stateVariable(S1,SV1),
   entity__isExplicitlyInstanceOf_stateVariable(S2,SV2),
   ~( entity__hasAnElementWhichIsNotSharedBy_entity(S,S1) ),
   ~( entity__hasAnElementWhichIsNotSharedBy_entity(S,S2) ),
   state__isNextInstanceOf_stateVariable(S,SV).
entity__i_stateVariable(S,SV) :-
   entityMappingClass__mapsTo_stateVariable(EMC,SV),
   class__isExplicitlySubclassOf_class(EMC,'union'),
   entityMappingClass__mapsFrom_stateVariable(EMC,SV1),
   entityMappingClass__mapsFrom_stateVariable(EMC,SV2),
   id__lt_id(SV1,SV2),
   entity__isExplicitlyInstanceOf_stateVariable(S1,SV1),
   entity__isExplicitlyInstanceOf_stateVariable(S2,SV2),
   ~( entity__hasAnElementWhichIsNotSharedBy_entity(S1,S) ),
   ~( entity__hasAnElementWhichIsNotSharedBy_entity(S2,S) ),
   state__isNextInstanceOf_stateVariable(S,SV).
entity__i_stateVariable(S,SV) :-
   entityMappingClass__mapsTo_stateVariable(EMC,SV),
   class__isExplicitlySubclassOf_class(EMC,'subtraction'),
   entityMappingClass__subtractsFrom_stateVariable(EMC,SV1),
   entityMappingClass__subtracts_stateVariable(EMC,SV2),
   id__lt_id(SV1,SV2),
   entity__isExplicitlyInstanceOf_stateVariable(S1,SV1),
   entity__isExplicitlyInstanceOf_stateVariable(S2,SV2),
   ~( entity__hasAnElementWhichIsNotSharedBy_entity(S,S1) ),
   ~( entity__hasAnElementWhichIsSharedBy_entity(S,S2) ),
   state__isNextInstanceOf_stateVariable(S,SV).


entityMappingClass__mapsFrom_stateVariable(EMC,SV) :-
   entityMappingClass__subtractsFrom_stateVariable(EMC,SV).
entityMappingClass__mapsFrom_stateVariable(EMC,SV) :-
   entityMappingClass__subtracts_stateVariable(EMC,SV).


entity__hasAnElementWhichIsNotSharedBy_entity(S1,S2) :-
   state__e_entity(ST,S1),
   ~( state__e_entity(ST,S2) ).

entity__hasAnElementWhichIsSharedBy_entity(S,S2) :-
   state__e_entity(ST,S),
   state__e_entity(ST,S2).


%:- consult('emc.prc').
%:- consult('emc.prp').
% ESMC.PRO: Abstractions about elementary state mapping classes.


eachInstanceOfMc__mapsFromAnInstanceOf_sv(ESMC,SV) :-
   elemStateMappingClass__mapsFrom_stateVariable(ESMC,SV).


eachInstanceOfMc__mapsToAnInstanceOf_sv(ESMC,SV) :-
   elemStateMappingClass__mapsTo_stateVariable(ESMC,SV).



%:- consult('esmc.prc').
%:- consult('esmc.prp').

% BMC.PRO: Abstractions about boolean mapping classes.


elemStateMappingClass__mapsFrom_stateVariable(BMC, BSV) :-
   booleanMc__mapsFrom_booleanSv(BMC, BSV).

elemStateMappingClass__mapsTo_stateVariable(BMC, BSV) :-
   booleanMc__mapsTo_booleanSv(BMC, BSV).


booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   state__isExplicitlyInstanceOf_stateVariable('TRUE', BSV1),         
   class__isExplicitlySubclassOf_class(BMC, '~'),
   BS = 'FALSE',
   state__isNextInstanceOf_stateVariable(BS, BSV).
booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   state__isExplicitlyInstanceOf_stateVariable('FALSE', BSV1),
   class__isExplicitlySubclassOf_class(BMC, '~'),
   BS = 'TRUE',
   state__isNextInstanceOf_stateVariable(BS, BSV).         
                                                               % NOT

                                                               
booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   booleanMc__mapsFrom_booleanSv(BMC, BSV2),
   id__lt_id(BSV1, BSV2),
   state__isExplicitlyInstanceOf_stateVariable('FALSE', BSV1),
   state__isExplicitlyInstanceOf_stateVariable('FALSE', BSV2),
   class__isExplicitlySubclassOf_class(BMC, '|'),
   BS = 'FALSE',
   state__isNextInstanceOf_stateVariable(BS, BSV).
booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   booleanMc__mapsFrom_booleanSv(BMC, BSV2),
   id__lt_id(BSV1, BSV2),
   state__isExplicitlyInstanceOf_stateVariable('TRUE', BSV1),
   state__isExplicitlyInstanceOf_stateVariable('FALSE', BSV2),
   class__isExplicitlySubclassOf_class(BMC, '|'),
   BS = 'TRUE',
   state__isNextInstanceOf_stateVariable(BS, BSV). 
booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   booleanMc__mapsFrom_booleanSv(BMC, BSV2),
   id__lt_id(BSV1, BSV2),
   state__isExplicitlyInstanceOf_stateVariable('FALSE', BSV1),
   state__isExplicitlyInstanceOf_stateVariable('TRUE', BSV2),
   class__isExplicitlySubclassOf_class(BMC, '|'),
   BS = 'TRUE',
   state__isNextInstanceOf_stateVariable(BS, BSV). 
booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   booleanMc__mapsFrom_booleanSv(BMC, BSV2),
   id__lt_id(BSV1, BSV2),
   state__isExplicitlyInstanceOf_stateVariable('TRUE', BSV1),
   state__isExplicitlyInstanceOf_stateVariable('TRUE', BSV2),
   class__isExplicitlySubclassOf_class(BMC, '|'),
   BS = 'TRUE',
   state__isNextInstanceOf_stateVariable(BS, BSV). 
                                                               % OR

                                                               
booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   booleanMc__mapsFrom_booleanSv(BMC, BSV2),
   id__lt_id(BSV1, BSV2),
   state__isExplicitlyInstanceOf_stateVariable('FALSE', BSV1),
   state__isExplicitlyInstanceOf_stateVariable('FALSE', BSV2),
   class__isExplicitlySubclassOf_class(BMC, '&'),
   BS = 'FALSE',
   state__isNextInstanceOf_stateVariable(BS, BSV).
booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   booleanMc__mapsFrom_booleanSv(BMC, BSV2),
   id__lt_id(BSV1, BSV2),
   state__isExplicitlyInstanceOf_stateVariable('TRUE', BSV1),
   state__isExplicitlyInstanceOf_stateVariable('FALSE', BSV2),
   class__isExplicitlySubclassOf_class(BMC, '&'),
   BS = 'FALSE',
   state__isNextInstanceOf_stateVariable(BS, BSV).
booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   booleanMc__mapsFrom_booleanSv(BMC, BSV2),
   id__lt_id(BSV1, BSV2),
   state__isExplicitlyInstanceOf_stateVariable('FALSE', BSV1),
   state__isExplicitlyInstanceOf_stateVariable('TRUE', BSV2),
   class__isExplicitlySubclassOf_class(BMC, '&'),
   BS = 'FALSE',
   state__isNextInstanceOf_stateVariable(BS, BSV).
booleanState__i_booleanSv(BS, BSV) :-
   nonvar(BSV),
   booleanMc__mapsTo_booleanSv(BMC, BSV),
   booleanMc__mapsFrom_booleanSv(BMC, BSV1),
   booleanMc__mapsFrom_booleanSv(BMC, BSV2),
   id__lt_id(BSV1, BSV2),
   state__isExplicitlyInstanceOf_stateVariable('TRUE', BSV1),
   state__isExplicitlyInstanceOf_stateVariable('TRUE', BSV2),
   class__isExplicitlySubclassOf_class(BMC, '&'),
   BS = 'TRUE',
   state__isNextInstanceOf_stateVariable(BS, BSV).
                                                               % AND



%:- consult('bmc.prc').
%:- consult('bmc.prp').
% NMC.PRO: Abstractions about numeric mapping classes.
% Author: Frans Meijs, 1992


state__i_stateVariable(N,NSV) :-
   number__i_numericSv(N,NSV).


concept__isExplicitlyAssociatedWith_concept(UM,NSV) :-
   unitOfMeasure__isAssociatedWith_numericSv(UM,NSV).


elemStateMappingClass__mapsFrom_stateVariable(NMC,NSV) :-
   numericMc__mapsFrom_numericSv(NMC,NSV).


elemStateMappingClass__mapsTo_stateVariable(NMC,NSV) :-
   numericMc__mapsTo_numericSv(NMC,NSV).


number__i_numericSv(N,NSV) :-
   nonvar(NSV),
   numericMc__mapsTo_numericSv(NMC,NSV),
   numericMc__mapsFrom_numericSv(NMC,NSV1),
   numericMc__mapsFrom_numericSv(NMC,NSV2),
   id__lt_id(NSV1,NSV2),
   stateVariable__isAssociatedWith_sort(NSV1,['SortOf','integerNumbers']),
   stateVariable__isAssociatedWith_sort(NSV2,['SortOf','integerNumbers']),
   state__isExplicitlyInstanceOf_stateVariable(N1,NSV1),
   state__isExplicitlyInstanceOf_stateVariable(N2,NSV2),
   class__isSubclassOf_class(NMC,'+'),
   int__plus_int__is_int(N1,N2,N),        % Predefined arithmetic of Prolog.
   state__isNextInstanceOf_stateVariable(N,NSV).
number__i_numericSv(N,NSV) :-
   nonvar(NSV),
   numericMc__mapsTo_numericSv(NMC,NSV),
   numericMc__mapsFrom_numericSv(NMC,NSV1),
   numericMc__mapsFrom_numericSv(NMC,NSV2),
   id__lt_id(NSV1,NSV2),
   stateVariable__isAssociatedWith_sort(NSV1,['SortOf','realNumbers']),
   stateVariable__isAssociatedWith_sort(NSV2,['SortOf','realNumbers']),
   state__isExplicitlyInstanceOf_stateVariable(N1,NSV1),
   state__isExplicitlyInstanceOf_stateVariable(N2,NSV2),
   class__isSubclassOf_class(NMC,'+'),
   real__plus_real__is_real(N1,N2,N),     % Predefined arithmetic of Prolog.
   state__isNextInstanceOf_stateVariable(N,NSV).
number__i_numericSv(N,NSV) :-
   nonvar(NSV),
   numericMc__mapsTo_numericSv(NMC,NSV),
   numericMc__mapsFrom_numericSv(NMC,NSV1),
   numericMc__mapsFrom_numericSv(NMC,NSV2),
   id__lt_id(NSV1,NSV2),
   stateVariable__isAssociatedWith_sort(NSV1,['SortOf','integerNumbers']),
   stateVariable__isAssociatedWith_sort(NSV2,['SortOf','integerNumbers']),
   state__isExplicitlyInstanceOf_stateVariable(N1,NSV1),
   state__isExplicitlyInstanceOf_stateVariable(N2,NSV2),
   class__isSubclassOf_class(NMC,'*'),
   int__times_int__is_int(N1,N2,N),        % Predefined arithmetic of Prolog.
   state__isNextInstanceOf_stateVariable(N,NSV).
number__i_numericSv(N,NSV) :-
   nonvar(NSV),
   numericMc__mapsTo_numericSv(NMC,NSV),
   numericMc__mapsFrom_numericSv(NMC,NSV1),
   numericMc__mapsFrom_numericSv(NMC,NSV2),
   id__lt_id(NSV1,NSV2),
   stateVariable__isAssociatedWith_sort(NSV1,['SortOf','realNumbers']),
   stateVariable__isAssociatedWith_sort(NSV2,['SortOf','realNumbers']),
   state__isExplicitlyInstanceOf_stateVariable(N1,NSV1),
   state__isExplicitlyInstanceOf_stateVariable(N2,NSV2),
   class__isSubclassOf_class(NMC,'*'),
   real__times_real__is_real(N1,N2,N),
                                    % Predefined arithmetic of Prolog.
   state__isNextInstanceOf_stateVariable(N,NSV).
number__i_numericSv(N,NSV) :-
   nonvar(NSV),
   numericMc__mapsTo_numericSv(NMC,NSV),
   subtractionClass__subtracts_numericSv(NMC,NSV1),
   subtractionClass__subtractsBy_numericSv(NMC,NSV2),
   stateVariable__isAssociatedWith_sort(NSV1,['SortOf','integerNumbers']),
   stateVariable__isAssociatedWith_sort(NSV2,['SortOf','integerNumbers']),
   state__isExplicitlyInstanceOf_stateVariable(N1,NSV1),
   state__isExplicitlyInstanceOf_stateVariable(N2,NSV2),
   class__isSubclassOf_class(NMC,'-'),
   int__minus_int__is_int(N1,N2,N),        % Predefined arithmetic of Prolog.
   state__isNextInstanceOf_stateVariable(N,NSV).
number__i_numericSv(N,NSV) :-
   nonvar(NSV),
   numericMc__mapsTo_numericSv(NMC,NSV),
   subtractionClass__subtracts_numericSv(NMC,NSV1),
   subtractionClass__subtractsBy_numericSv(NMC,NSV2),
   stateVariable__isAssociatedWith_sort(NSV1,['SortOf','realNumbers']),
   stateVariable__isAssociatedWith_sort(NSV2,['SortOf','realNumbers']),
   state__isExplicitlyInstanceOf_stateVariable(N1,NSV1),
   state__isExplicitlyInstanceOf_stateVariable(N2,NSV2),
   class__isSubclassOf_class(NMC,'-'),
   real__minus_real__is_real(N1,N2,N),
                                     % Predefined arithmetic of Prolog.
   state__isNextInstanceOf_stateVariable(N,NSV).
number__i_numericSv(N,NSV) :-
   nonvar(NSV),
   numericMc__mapsTo_numericSv(NMC,NSV),
   subtractionClass__divides_numericSv(NMC,NSV1),
   subtractionClass__dividesBy_numericSv(NMC,NSV2),
   stateVariable__isAssociatedWith_sort(NSV1,['SortOf','integerNumbers']),
   stateVariable__isAssociatedWith_sort(NSV2,['SortOf','integerNumbers']),
   state__isExplicitlyInstanceOf_stateVariable(N1,NSV1),
   state__isExplicitlyInstanceOf_stateVariable(N2,NSV2),
   class__isSubclassOf_class(NMC,'div'),
   int__div_int__is_integer(N1,N2,N),  % Predefined arithmetic of Prolog.
   state__isNextInstanceOf_stateVariable(N,NSV).
number__i_numericSv(N,NSV) :-
   nonvar(NSV),
   numericMc__mapsTo_numericSv(NMC,NSV),
   subtractionClass__divides_numericSv(NMC,NSV1),
   subtractionClass__dividesBy_numericSv(NMC,NSV2),
   stateVariable__isAssociatedWith_sort(NSV1,['SortOf','realNumbers']),
   stateVariable__isAssociatedWith_sort(NSV2,['SortOf','realNumbers']),
   state__isExplicitlyInstanceOf_stateVariable(N1,NSV1),
   state__isExplicitlyInstanceOf_stateVariable(N2,NSV2),
   class__isSubclassOf_class(NMC,'/'),
   real__dividedBy_real__is_real(N1,N2,N),
                                     % Predefined arithmetic of Prolog.
   state__isNextInstanceOf_stateVariable(N,NSV).


unitOfMeasure__isAssociatedWith_numericSv(UM,NSV) :-
   unitOfMeasure__isExplicitlyAssociatedWith_numericSv(UM,NSV).
unitOfMeasure__isAssociatedWith_numericSv(UM,NSV) :-
   UM = [UM1|[NMCsymbol|[UM2]]],
   nonvar(NSV),
   numericMc__mapsTo_numericSv(NMC,NSV),
   numericMc__mapsFrom_numericSv(NMC,NSV1),
   unitOfMeasure__isAssociatedWith_numericSv(UM1,NSV1),
   numericMc__mapsFrom_numericSv(NMC,NSV2),
   unitOfMeasure__isAssociatedWith_numericSv(UM2,NSV2),
   class__isSubclassOf_class(NMC,NMCsymbol).
            % Special case of division T.B.D.


numericMc__mapsFrom_numericSv(SC,NSV) :-
   subtractionClass__subtracts_numericSv(SC,NSV).
numericMc__mapsFrom_numericSv(SC,NSV) :-
   subtractionClass__subtractsBy_numericSv(SC,NSV).
numericMc__mapsFrom_numericSv(DC,DV) :-
   divisionClass__divides_numericSv(DC,DV).
numericMc__mapsFrom_numericSv(DC,DV) :-
   divisionClass__dividesBy_numericSv(DC,DV).

   
%:- consult('nmc.prp').
%:- consult('nmc.prc').
%:- consult('int.pro').
%:- consult('nat.pro').
% SU.PRO: Abstractions about structural units.
% Author: Frans Meijs, 1992


entity(E) :- structuralUnit(E).


entity__e_entity(E1,E2) :-
   behaviouralUnit__e_su(E1,E2).
entity__e_entity(E1,E2) :-
   structuralUnit__e_su(E1,E2).


structuralUnit__e_su(SU1,SU2) :-
   structuralUnit__isPrivateElementOf_su(SU1,SU2).
structuralUnit__e_su(SU1,SU2) :-
   structuralUnit__isShareableElementOf_su(SU1,SU2).


structuralUnit__isPrivateElementOf_su(SU1,SU2) :-
   structuralUnit__isExplicitlyPrivateElementOf_su(SU1,SU2).
structuralUnit__isPrivateElementOf_su(SU1,SU2) :-
   nonvar(SU1),
   structuralUnit__isExplicitlyPrivateElementOf_su(SU1,SU3),
   structuralUnit__isPrivateElementOf_su(SU3,SU2).
structuralUnit__isPrivateElementOf_su(SU1,SU2) :-
   var(SU1), nonvar(SU2),
   structuralUnit__isExplicitlyPrivateElementOf_su(SU3,SU2),
   structuralUnit__isPrivateElementOf_su(SU1,SU3).

structuralUnit__isShareableElementOf_su(SU1,SU2) :-
   structuralUnit__isExplicitlyShareableElementOf_su(SU1,SU2).
   
eachInstanceOfClass__isExplicitlyElementOf_set(SV,SU) :-
   stateVariable__e_su(SV,SU).


stateVariable__e_su(SV,SU) :-
   stateVariable__isPrivateElementOf_su(SV,SU).
stateVariable__e_su(SV,SU) :-
   stateVariable__isShareableElementOf_su(SV,SU).
stateVariable__e_su(SV,SU) :-
   stateVariable__isInputElementOf_su(SV,SU).
stateVariable__e_su(SV,SU) :-
   stateVariable__isOutputElementOf_su(SV,SU).


stateVariable__e_bu(SV,BU) :-
   stateVariable__isShareableElementOf_bu(SV,BU).
stateVariable__e_bu(SV,BU) :-
   stateVariable__isInputElementOf_bu(SV,BU).
stateVariable__e_bu(SV,BU) :-
   stateVariable__isOutputElementOf_bu(SV,BU).
                                         % Note 2.


stateVariable__isShareableElementOf_bu(SV,BU) :-
   stateVariable__isPrivateElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).   
stateVariable__isShareableElementOf_bu(SV,BU) :-
   stateVariable__isShareableElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).


stateVariable__isInputElementOf_bu(SV,BU) :-
   nonvar(SV),
   stateVariable__isInputElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).
stateVariable__isInputElementOf_bu(SV,BU) :-
   var(SV),nonvar(BU),
   behaviouralUnit__e_su(BU,SU),
   stateVariable__isInputElementOf_su(SV,SU).
stateVariable__isInputElementOf_bu(SV,BU) :-
   nonvar(SV),
   stateVariable__isPrivateElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).
stateVariable__isInputElementOf_bu(SV,BU) :-
   var(SV),nonvar(BU),
   behaviouralUnit__e_su(BU,SU),
   stateVariable__isPrivateElementOf_su(SV,SU).
   

stateVariable__isOutputElementOf_bu(SV,BU) :-
   nonvar(SV),
   stateVariable__isOutputElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).   
stateVariable__isOutputElementOf_bu(SV,BU) :-
   var(SV),nonvar(BU),
   behaviouralUnit__e_su(BU,SU),
   stateVariable__isOutputElementOf_su(SV,SU).      % Note 2.
stateVariable__isOutputElementOf_bu(SV,BU) :-
   nonvar(SV),
   stateVariable__isPrivateElementOf_su(SV,SU),
   behaviouralUnit__e_su(BU,SU).
stateVariable__isOutputElementOf_bu(SV,BU) :-
   var(SV),nonvar(BU),
   behaviouralUnit__e_su(BU,SU),
   stateVariable__isPrivateElementOf_su(SV,SU).


stateVariable__isOutputElementOf_su(SV,SU) :-
   nonvar(SV),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2), % Note 1.
   structuralUnit__isOutputElementOf_su(SU2,SU).
stateVariable__isOutputElementOf_su(SV,SU) :-
   var(SV),nonvar(SU),
   structuralUnit__isOutputElementOf_su(SU2,SU),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2).


stateVariable__isPrivateElementOf_su(SV,SU) :-
   stateVariable__isExplicitlyPrivateElementOf_su(SV,SU).
stateVariable__isPrivateElementOf_su(SV,SU) :-
   nonvar(SV),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2), % Note 1.
   structuralUnit__isPrivateElementOf_su(SU2,SU).
stateVariable__isPrivateElementOf_su(SV,SU) :-
   var(SV),nonvar(SU),
   structuralUnit__isPrivateElementOf_su(SU2,SU),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2).


stateVariable__isShareableElementOf_su(SV,SU) :-
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU).
stateVariable__isShareableElementOf_su(SV,SU) :-
   nonvar(SV),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2), % Note 1.
   structuralUnit__isShareableElementOf_su(SU2,SU).
stateVariable__isShareableElementOf_su(SV,SU) :-
   var(SV),nonvar(SU),
   structuralUnit__isShareableElementOf_su(SU2,SU),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2).


stateVariable__isInputElementOf_su(SV,SU) :-
   nonvar(SV),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2), % Note 1.
   structuralUnit__isInputElementOf_su(SU2,SU).
stateVariable__isInputElementOf_su(SV,SU) :-
   var(SV),nonvar(SU),
   structuralUnit__isInputElementOf_su(SU2,SU),
   stateVariable__isExplicitlyShareableElementOf_su(SV,SU2).



% Notes:

% 1.  NOT "_e__", because that would cause infinite recursion.
% 2. All "private/input/output" defs involving both SU-s and BU-s should
%     be confined to this file.
% SYMBOL.PRO: Abstractions about symbols.
% Author: Frans Meijs, 14 mar 1998.

association(Z) :- definition(Z).
structuralUnit(Z) :- symbol(Z).

concept__isAssociatedWith_concept(Z,Y) :-
   symbol__represents_concept(Z,Y).

definition(D) :-
   D = [Z|[Y]],
   symbol__represents_concept(Z,Y).

symbol__represents_concept(Z,Y) :-
   symbol__represents_association(Z,Y).

structuralUnit__isExplicitlyPrivateElementOf_su(Z,Y) :-
   symbol__e_symbol(Z,Y).



%:- consult('symbol.prc').


% PSVS.PRO: Abstractions about Passive State Variable Sets.
% Author: Frans Meijs, 1996


structuralUnit(SU) :- psvs(SU).


structuralUnit__isExplicitlyPrivateElementOf_su(SU1,SU2) :-
   psvs__isExplicitlyPrivateElementOf_su(SU1,SU2).


structuralUnit__isShareableElementOf_su(SU1,SU2) :-
   psvs__isShareableElementOf_su(SU1,SU2).


structuralUnit__isInputElementOf_su(SU1,SU2) :-
   psvs__isInputElementOf_su(SU1,SU2).

   
structuralUnit__isOutputElementOf_su(SU1,SU2) :-
   psvs__isOutputElementOf_su(SU1,SU2).



%:- consult('psvs.prc').
% MSGV.PRO: Abstractions about message- and signalling variables.
% Author: Frans A. Meijs, 1996.

psvs(MSGV) :- msgv(MSGV).

stateVariable__isExplicitlyAssociatedWith_sort(SEV,S) :-
   class__isSubclassOf_class(SEV, 'SignallingEventVariable'),
   S = ['SortOf','booleanStates'].

class__isExplicitlySubclassOf_class(SEV,'MsgSignallingEventVariable') :-
   exists( sev__e_msgv(SEV,_) ).

class__isExplicitlySubclassOf_class(DV,'DataVariable') :-
   exists( dataVariable__e_msgv(DV,_) ).

stateVariable__isExplicitlyShareableElementOf_su(DV,MSGV) :-
   dataVariable__e_msgv(DV,MSGV).
stateVariable__isExplicitlyShareableElementOf_su(SEV,MSGV) :-
   sev__e_msgv(SEV,MSGV).

psvs__isExplicitlyPrivateElementOf_su(MSGV,SU) :-
   msgv__isExplicitlyPrivateElementOf_su(MSGV,SU).

psvs__isInputElementOf_su(MSGV,SU) :-
   msgv__isInputElementOf_su(MSGV,SU).

psvs__isOutputElementOf_su(MSGV,SU) :-
   msgv__isOutputElementOf_su(MSGV,SU).


sev__e_msgv(SEV,MSGV) :-
   nonvar(SEV),
   sevId__eq_msgvId(SEV,MSGV),
   msgv(MSGV).
sev__e_msgv(SEV,MSGV) :-
   var(SEV),
   msgv(MSGV),
   sevId__eq_msgvId(SEV,MSGV).


   % Shorthand notation for MSGV-s.


msgv__isInputElementOf_su(MSGV,SU) :-
   msgv__isElementaryElementOf_msgv(MSGV,MSGV2),
   msgv__isInputElementOf_su(MSGV2,SU).
   

msgv__isOutputElementOf_su(MSGV,SU) :-
   msgv__isElementaryElementOf_msgv(MSGV,MSGV2),
   msgv__isOutputElementOf_su(MSGV2,SU).
   

msgv__isElementaryElementOf_msgv(MSGV1,MSGV2) :-
   msgv__e_msgv(MSGV1,MSGV2),
   ~( msgv__isExplicitlyElementOf_msgv(_,MSGV1) ).


structuralUnit__isExplicitlyPrivateElementOf_su(SU1,SU2) :-
   msgv__isExplicitlyElementOf_msgv(SU1,SU2).   % For constraint SU31.


msgv__e_msgv(C1,C2) :-
   msgv__isExplicitlyElementOf_msgv(C1,C2).
msgv__e_msgv(C1,C2) :-
   msgv__isExplicitlyElementOf_msgv(C1,C3),
   msgv__e_msgv(C3,C2).


   
%:- consult('msgv.prp').
%:- consult('msgv.prq').
%:- consult('msgv.prc').
% PROCESS.PRO: Abstractions about processes.

connection(M) :- mapping(M).
hierarchy(P) :- process(P).

connection__e_network(M, P) :-
   mapping__e_process(M, P).


connection__connectsFrom_node(M, S) :-
   mapping__mapsFrom_state(M, S).
   

connection__connectsTo_node(M, S) :-
   mapping__mapsTo_state(M, S).
% SORT.PRO: Abstractions about sorts.
% Author: Frans Meijs, 1992

set(D) :- sort(D).

set__intersects_set(S1,S2) :-
   sort__intersects_sort(S1,S2).

concept__isExplicitlyElementOf_set(ES,D) :-
   elemState__e_sort(ES,D).


set__isExplicitlyElementOf_set(S1,S2) :-
   sort__isExplicitlyElementOf_sort(S1,S2).
                  % set__e_set(S1,S2) :- sort__e_sort(S1,S2).
                  %     is covered by this clause.
set__isExplicitlyElementOf_set(S1,S2) :-
   entity__e_sort(S1,S2).


sort__e_sort(S1,S2) :-
   sort__isExplicitlyElementOf_sort(S1,S2).
sort__e_sort(S1,S2) :-
   sort__isExplicitlyElementOf_sort(S1,S3),
   sort__e_sort(S3,S2).


% MSGV.PRQ: Equivalences.
% Author: Frans Meijs, mar 1998


sevId__eq_msgvId(SEV,MSGV) :-
   ~( arityOfVars__gt_1(SEV,MSGV) ),
   SEV = ['SEV',MSGV].

