% CONCEPT.PRO: The theory with the highest level of abstraction.
% Author: Frans Meijs, 1990


concept(C) :- contradiction(C).        % For verification purposes.
concept(C) :- association(C).


association(A) :-
   nonvar(A),
   A = [C1|[C2]],
   concept__isAssociatedWith_concept(C1,C2).
association(A) :-
   var(A),
   concept__isAssociatedWith_concept(C1,C2),
   A = [C1,C2].


concept__isAssociatedWith_concept(C1,C2) :-
   concept__isExplicitlyAssociatedWith_concept(C1,C2).
         % Association is directed from subject noun to object noun, so
         %    concept__isExplicitlyAssociatedWith_concept(C2,C1) :-
         %                  concept__isAssociatedWith_concept(C1,C2).
         % is NOT true.
concept__isAssociatedWith_concept(C1,C2) :-
   concept__isExplicitlyAssociatedWith_concept(C1,C3),
   concept__isAssociatedWith_concept(C3,C2).
         % Because it is the most general closure, this clause is unrestricted w.r.t.
         % binding of C1 and C2.


%:- consult('concept.prc').
% CLASS.PRC

contradiction(['Contrad. class CLASS1: Concept', X,
   'seems to be a class but is not declared as such']) :-
   o_verifyingLogMsg('CLASS1'),
   class__isExplicitlySubclassOf_class(_, X),
   ~( class(X) ).
      % This is the first part of the Referential Integrity Constraint
      % from data base theory.


contradiction(['Contrad. class CLASS2: Concept', X,
   'seems to be a class but is not declared as such']) :-
   o_verifyingLogMsg('CLASS2'),
   class__isExplicitlySubclassOf_class(X, _),
   ~( class(X) ).
      % This is the second part of the Referential Integrity Constraint
      % from relational data base theory.
      % Classes are required to be specified explicitly. The 
      % alternative, the introduction of the specification  
      %    "class(C) :- class__isExplicitlyElementOf_class(C,_)." 
      % may cause duplicate solutions, and 
      % hence may increase verification time considerably.


contradiction(
   ['Contrad. class CLASS4', C1,
   'and', C2,
   'are both root in the classification tree']) :-
   o_verifyingLogMsg('CLASS4'),
   class(C1),
   ~( class__isExplicitlySubclassOf_class(C1, _) ),
   class(C2),
   id__lt_id(C1, C2),
   ~( class__isExplicitlySubclassOf_class(C2, _) ).

 

% CLASS.PRO: Abstractions about classes without explicit 
% identification of their instances.
% Author: Frans Meijs, 1990

concept(S) :- class(S).

concept__isExplicitlyAssociatedWith_concept(C1,C2) :-
   class__isSubclassOf_class(C1,C2).


class__isSubclassOf_class(C,C2) :- 
   class__isExplicitlySubclassOf_class(C,C2).
class__isSubclassOf_class(C1,C2) :-
   class__isExplicitlySubclassOf_class(C1,C3),
   C1 \= C3,            % Avoid infinite loop in case of erratic spec.
   class__isSubclassOf_class(C3,C2). 
               % It is assumed that there are no cycles (See CLASS.PRC).


association(A) :-
   nonvar(A),
   A = [C1|[C2]],
   class__isElementarySubclassOf_class(C1,C2).
association(A) :-
   var(A),
   class__isElementarySubclassOf_class(C1,C2),
   A = [C1,C2].
            % class__isElementarySubclassOf_class/2 is an 
            % auxiliary predicate.
            % For that reason it is not made subclass of 
            % concept__isExplicitlyAssociatedWith_concept/2.


class__isElementarySubclassOf_class(C1,C2) :-
   class__isSubclassOf_class(C1,C2),
   ~( class__isExplicitlySubclassOf_class(_,C1) ).



%:- consult('class.prc'). 
%:- consult('class.prp'). 


